@@ -3,7 +3,6 @@ import hashlib
 import requests
 import tempfile
 import mimetypes
-import numpy as np
 
 from PIL import Image
 from io import BytesIO
@@ -12,7 +11,6 @@ from datetime import datetime
 
 from . import config
 
-
 API_URL = 'https://api.abraia.me'
 tempdir = tempfile.gettempdir()
 
@@ -41,19 +39,10 @@ class APIError(Exception):
 
 
 class Abraia:
-    def __init__(self, folder=''):
-        self.auth = config.load_auth()
-        self.userid = self.load_user().get('id')
-        self.folder = folder
-
-    def load_user(self):
-        if self.auth[0] and self.auth[1]:
-            url = f"{API_URL}/users"
-            resp = requests.get(url, auth=self.auth)
-            if resp.status_code != 200:
-                raise APIError(resp.text, resp.status_code)
-            return resp.json()['user']
-        return {}
+    def __init__(self):
+        abraia_id, abraia_key = config.load()
+        self.auth = config.load_auth(abraia_key)
+        self.userid = abraia_id
 
     def list_files(self, path=''):
         dirname = os.path.dirname(path)
@@ -164,23 +153,20 @@ class Abraia:
         except:
             return stream
 
-    def load_image(self, path):
-        stream = self.download_file(path)
-        return np.asarray(Image.open(stream))
-
     def save_file(self, path, stream):
         stream =  BytesIO(bytes(stream, 'utf-8')) if isinstance(stream, str) else stream
         return self.upload_file(stream, path)
 
-    def save_image(self, path, img):
-        # stream = BytesIO()
-        # mime = mimetypes.guess_type(path)[0]
-        # format = mime.split('/')[1]
-        # Image.fromarray(img).save(stream, format)
-        # print(mime, format)
-        basename = os.path.basename(path)
-        src = os.path.join(tempdir, basename)
-        Image.fromarray(img).save(src)
+    def load_image(self, path):
+        dest = os.path.join(tempdir, path)
+        os.makedirs(os.path.dirname(dest), exist_ok=True)
+        self.download_file(path, dest)
+        return Image.open(dest)
+
+    def save_image(self, path, im):
+        src = os.path.join(tempdir, path)
+        os.makedirs(os.path.dirname(src), exist_ok=True)
+        im.save(src)
         return self.upload_file(src, path)
 
     def capture_text(self, path):
@@ -198,4 +184,11 @@ class Abraia:
             raise APIError(resp.text, resp.status_code)
         labels = resp.json().get('Labels')
         return [l.get('Name') for l in labels]
-    
\ No newline at end of file
+    
+    def detect_faces(self, path):
+        url = f"{API_URL}/rekognition/{self.userid}/{path}"
+        resp = requests.get(url, params={'mode': 'faces'}, auth=self.auth)
+        if resp.status_code != 200:
+            raise APIError(resp.text, resp.status_code)
+        faces = resp.json().get('Faces')
+        return faces
\ No newline at end of file
